#include <iostream>
#include <string>
#include <memory>
#include <exception>

class Not_enough_money : public std::exception {
  public:
	const char* what() const noexcept override {
		return "Account balance is too low to execute withdrawal";
	}
};


// ...


std::unique_ptr<Bank_account> create_account_a() {
	return std::make_unique<Giro_account>("Max Musterman", 100);
}

int main() {
	auto a = create_account_a();
	auto b = Bank_account{"Tim"};

	a->deposit(50);
	a->deposit(10);
	b.deposit(50);
	b.deposit(10);
	
	// Erwartete Ausgabe:
	// Max Musterman : 60
	// Tim : 60
	a->print();
	b.print();
	
	// Erwartete Ausgabe (auf stderr):
	// Account balance is too low to execute withdrawal
	try {
		a->withdraw(50);
		a->withdraw(30);
		
	} catch(const std::exception& e) {
		std::cerr << e.what() << '\n';
	}
	try {
		b.withdraw(50);
		b.withdraw(30);
	
	} catch(const std::exception& e) {
		std::cerr << e.what() << '\n';
	}
	
	// Erwartete Ausgabe:
	// Max Musterman : -20
	// Tim : 10
	a->print();
	b.print();
}

