#include "circle.hpp"
#include "rectangle.hpp"
#include "triangle.hpp"
#include "renderer.hpp"
#include "shape.hpp"

#include <iostream>
#include <memory>
#include <vector>

int main()
{
	// Zuerst erzeugen wir uns einige lokale Variablen.
	// Hier ist der Typ der Variablen der selbe wie der dynamische Typ, weswegen wir sie wie
	//   bisher als normale lokale Variablen speichern können.
	auto circle    = shapes::Circle(0, 0, shapes::Color::red, 2.f);
	auto rectangle = shapes::Rectangle(1, 2, shapes::Color::green, 3.f, 4.f);
	auto triangle  = shapes::Triangle(1, 2, shapes::Color::blue, 2.f, 4.f, 3.f);
	
	// Wir können die Objekte auch nach wie vor ganz normal kopieren, sofern wir eine der
	//   konkreten Klassen haben und nicht nur die rein virtuelle Basisklasse.
	auto circle_2  = circle;
	
	// Und natürlich können wir auf den Objekten auch ganz normal die Member Funktione aufrufen.
	std::cout << "Circle area    : " << circle.area() << "\n";
	std::cout << "Rectangle area : " << rectangle.area() << "\n";
	std::cout << "Circle_2 area  : " << circle_2.area() << "\n";
	std::cout << "Triangle area  : " << triangle.area() << "\n";
	std::cout << "\n";
	
	
	// Wenn sich der statische vom dynamischen Typen unterscheidet brauchen wir allerdings std::unique_ptr.
	// Das ist z.B. der Fall wenn wir (wie hier) einen Container haben wollen, der Objekte mit
	//   unterschiedlichen dynamischen Typen aber einer gemeinsamen Basisklasse enthält.
	auto shapes = std::vector<std::unique_ptr<shapes::Shape>>{};
	
	// Die std::unique_ptr können wir z.B. mit std::make_unique<T> erzeugen und auch direkt
	//   die Konstruktor-Parameter übergeben.
	shapes.push_back(std::make_unique<shapes::Circle>(3.f, -2.f, shapes::Color::blue, 1.f));
	shapes.push_back(std::make_unique<shapes::Rectangle>(1.f, 2.f, shapes::Color::green, 3.f, 4.f));
	shapes.push_back(std::make_unique<shapes::Circle>(0.f, -6.f, shapes::Color::blue, 1.f));
	shapes.push_back(std::make_unique<shapes::Circle>(8.f, 3.f, shapes::Color::blue, 1.f));
	shapes.push_back(std::make_unique<shapes::Triangle>(-2.f, 2.f, shapes::Color::red, 3.f, 4.f, 2.f));

	// Wenn wir die std::unique_ptr verwenden müssen wir nur daran denken, dass wir sie wie normale
	//   Pointer dereferenzieren müssen und das sie potentiell auch nullptr sein könnten.
	auto i = 0;
	for(auto& shape : shapes) {
		std::cout << "Shape " << i++ << "\n";
		std::cout << "  Area          : " << shape->area() << "\n";
		std::cout << "  Circumference : " << shape->circumference() << "\n";
	}
	std::cout << "\n";
	
	
	auto renderer = shapes::create_renderer();
	
	renderer->draw(circle);
	renderer->draw(rectangle);
	renderer->draw(circle_2);
	renderer->draw(triangle);
}
