#pragma once

#include <memory>

namespace shapes {

	// Eine Forward-Declaration reicht hier aus, weil wir den Typen nur
	//   benutzen um einen Referenz-Parameter zu deklarieren.
	class Shape;

	// Hier haben wir einen häufigen Anwendungsfall für Vererbung:
	// Wir definieren uns eine allgemeine Schnittstelle für einen Renderer, der Shapes zeichen kann
	//   und für den wir später eine konkrete Implementierung definieren können, ohne das der restliche
	//   Code die konkrete Implementierung kennen muss.
	class Renderer {
	  public:
		// Weil das hier eine Schnittstelle für einen relativ komplexen Typen werden soll, definieren
		//   wir die Special Member Functions so, dass der Typ weder kopierbar noch verschiebbar ist.
		Renderer() = default;
		Renderer(const Renderer&) = delete;
		Renderer(Renderer&&) = delete;
		Renderer& operator=(const Renderer&) = delete;
		Renderer& operator=(Renderer&&) = delete;
		virtual ~Renderer() = default;

		// Und wir definieren eine pure virtual Member Funktion für die eigentliche Schnittstelle.
		virtual void draw(const Shape& renderer) = 0;
	};

	// Damit wir eine Instanz erzeugen können, ohne das der restliche Code die Implementierung sehen kann,
	//   definieren wir außerdem noch eine Factory-Funktion, die uns einen std::unique_ptr<Renderer> erzeugt.
	extern std::unique_ptr<Renderer> create_renderer();

} // namespace shapes
