#pragma once

namespace shapes {

	// Eine Forward-Declaration reicht hier aus, weil wir den Typen nur
	//   benutzen um einen Referenz-Parameter zu deklarieren.
	class Shape_visitor;

	enum class Color { red, green, blue };

	class Shape {
	  public:
		Shape(float x, float y, Color color)
		  : x_(x), y_(y), color_(color) {}
		// Der Destruktor sollte virtual sein um zu garantieren das die Destruktoren der
		//   abgeleiteten Klassen auch dann aufgerufen werden, wenn wir das Objekt über
		//   einen std::unique_ptr<Shape> zerstören.
		// Das ist idR. zu empfehlen, wenn wir Vererbung verwenden und vorallem wenn wir
		//   auch noch andere virtual Methoden haben
		virtual ~Shape() = default;

		// pure virtual, müssen also in abgeleiteten Klassen implementiert werden
		virtual float area() const          = 0;
		virtual float circumference() const = 0;
		
		float x()const {return x_;}
		float y()const {return y_;}
		Color color()const {return color_;}

		// Startpunkt für das Visitor-Pattern.
		// Die konkreten Klassen, die von Shape ableiten, müssen diese Methode implementieren und die
		//   entsprechende Methode für ihren tatsächlichen Typen in Shape_visitor aufrufen.
		virtual void visit(Shape_visitor&) const = 0;

	  protected:
		// Wir definieren die Special Member Functions für Shape hier als protected
		//   damit man sie nicht unabsichtlich auf der Basisklasse aufrufen kann (Slicing).
		// In den abgeleiteten Klassen definieren wie sie dann nicht mehr,
		//   wodurch dort wieder automatisch Implementierungen generiert, die public sind.
		Shape(const Shape&) = default;
		Shape(Shape&&)      = default;

		Shape& operator=(const Shape&) = default;
		Shape& operator=(Shape&&) = default;

	  private:
		float x_, y_;
		Color color_;
	};

} // namespace shapes
