#pragma once

namespace shapes {

	// Eine Forward-Declaration reicht hier aus, weil wir die Typen nur
	//   benutzen um Referenz-Parameter zu deklarieren.
	class Circle;
	class Rectangle;

	// Schnittstelle für das Visitor-Pattern (siehe renderer.cpp für Implementierung).
	// Kann von anderen Klassen implementiert werden um Code abhängig vom dynamischen Typen
	//   einer Shape&-Variable auszuführen (Double-Dispatch).
	//
	// Mehr Details zu dieser Vorgehensweise findet ihr u.a. hier:
	//   https://gieseanw.wordpress.com/2018/12/29/stop-reimplementing-the-virtual-table-and-start-using-double-dispatch/
	class Shape_visitor {
	  public:
	  	Shape_visitor() = default;
		virtual ~Shape_visitor();
		
		// Hier definieren wir eine pure virtual Methode für jeden möglichen Shape-Typen
	  	virtual void visit(const Circle&) = 0;
	  	virtual void visit(const Rectangle&) = 0;
	  	
	  protected:
	  	Shape_visitor(const Shape_visitor&) = default;
	  	Shape_visitor(Shape_visitor&&) = default;
	  	Shape_visitor& operator=(const Shape_visitor&) = default;
	  	Shape_visitor& operator=(Shape_visitor&&) = default;
	};
	
} // namespace shapes
